@@ -39,29 +39,32 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import pyatspi
-
+from . import ax_event_synthesizer
+from . import action_presenter
 from . import braille_generator
 from . import debug
 from . import event_manager
+from . import flat_review_presenter
 from . import formatting
-from . import label_inference
 from . import keybindings
-from . import orca_state
-from . import script_manager
+from . import label_inference
+from . import learn_mode_presenter
+from . import mouse_review
+from . import notification_presenter
+from . import object_navigator
 from . import script_utilities
-from . import settings
 from . import settings_manager
+from . import sleep_mode_manager
 from . import sound_generator
+from . import speech_and_verbosity_manager
 from . import speech_generator
 from . import structural_navigation
-from . import where_am_I
+from . import system_information_presenter
+from . import table_navigator
 from . import bookmarks
-from . import tutorialgenerator
+from . import where_am_i_presenter
+from .ax_object import AXObject
 
-_eventManager = event_manager.getManager()
-_scriptManager = script_manager.getManager()
-_settingsManager = settings_manager.getManager()
 
 class Script:
     """The specific focus tracking scripts for applications.
@@ -78,12 +81,7 @@ class Script:
         self.app = app
 
         if app:
-            try:
-                self.name = self.app.name
-            except (LookupError, RuntimeError):
-                msg = 'ERROR: Could not get name of script app %s'
-                debug.println(debug.LEVEL_INFO, msg, True)
-                self.name = "default"
+            self.name = AXObject.get_name(self.app) or "default"
         else:
             self.name = "default"
 
@@ -101,12 +99,24 @@ class Script:
         self.caretNavigation = self.getCaretNavigation()
         self.bookmarks = self.getBookmarks()
         self.liveRegionManager = self.getLiveRegionManager()
+        self.notificationPresenter = self.getNotificationPresenter()
+        self.flatReviewPresenter = self.getFlatReviewPresenter()
+        self.speechAndVerbosityManager = self.getSpeechAndVerbosityManager()
+        self.systemInformationPresenter = self.getSystemInformationPresenter()
+        self.objectNavigator = self.getObjectNavigator()
+        self.tableNavigator = self.getTableNavigator()
+        self.whereAmIPresenter = self.getWhereAmIPresenter()
+        self.learnModePresenter = self.getLearnModePresenter()
+        self.mouseReviewer = self.getMouseReviewer()
+        self.eventSynthesizer = self.getEventSynthesizer()
+        self.actionPresenter = self.getActionPresenter()
+        self.sleepModeManager = self.getSleepModeManager()
 
         self.chat = self.getChat()
         self.inputEventHandlers = {}
         self.pointOfReference = {}
         self.setupInputEventHandlers()
-        self.keyBindings = self.getKeyBindings()
+        self.keyBindings = keybindings.KeyBindings()
         self.brailleBindings = self.getBrailleBindings()
 
         self.formatting = self.getFormatting()
@@ -115,15 +125,15 @@ class Script:
         self.speechGenerator = self.getSpeechGenerator()
         self.generatorCache = {}
         self.eventCache = {}
-        self.whereAmI = self.getWhereAmI()
         self.spellcheck = self.getSpellCheck()
-        self.tutorialGenerator = self.getTutorialGenerator()
 
         self.findCommandRun = False
-        self._lastCommandWasStructNav = False
 
-        msg = 'SCRIPT: %s initialized' % self.name
-        debug.println(debug.LEVEL_INFO, msg, True)
+        msg = f'SCRIPT: {self.name} initialized'
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
+    def __str__(self):
+        return f"{self.name}"
 
     def getListeners(self):
         """Sets up the AT-SPI event listeners for this script.
@@ -138,7 +148,7 @@ class Script:
         called by the key and braille bindings."""
         pass
 
-    def getKeyBindings(self):
+    def getKeyBindings(self, enabledOnly=True):
         """Defines the key bindings for this script.
 
         Returns an instance of keybindings.KeyBindings.
@@ -181,11 +191,6 @@ class Script:
         """
         return speech_generator.SpeechGenerator(self)
 
-    def getTutorialGenerator(self):
-        """Returns the tutorial generator for this script.
-        """
-        return tutorialgenerator.TutorialGenerator(self)
-
     def getChat(self):
         """Returns the 'chat' class for this script.
         """
@@ -200,8 +205,7 @@ class Script:
         return None
 
     def getUtilities(self):
-        """Returns the utilites for this script.
-        """
+        """Returns the utilities for this script."""
         return script_utilities.Utilities(self)
 
     def getLabelInference(self):
@@ -217,26 +221,48 @@ class Script:
     def getStructuralNavigation(self):
         """Returns the 'structural navigation' class for this script."""
         types = self.getEnabledStructuralNavigationTypes()
-        enable = _settingsManager.getSetting('structuralNavigationEnabled')
+        enable = settings_manager.getManager().getSetting('structuralNavigationEnabled')
         return structural_navigation.StructuralNavigation(self, types, enable)
 
     def getLiveRegionManager(self):
         """Returns the live region support for this script."""
         return None
 
-    def useStructuralNavigationModel(self):
-        """Returns True if we should use structural navigation. Most
-        scripts will have no need to override this.  Gecko does however
-        because within an HTML document there are times when we do want
-        to use it and times when we don't even though it is enabled,
-        e.g. in a form field.
-        """
-        return self.structuralNavigation.enabled
+    def getNotificationPresenter(self):
+        return notification_presenter.getPresenter()
 
-    def getWhereAmI(self):
-        """Returns the "where am I" class for this script.
-        """
-        return where_am_I.WhereAmI(self)
+    def getFlatReviewPresenter(self):
+        return flat_review_presenter.getPresenter()
+
+    def getSystemInformationPresenter(self):
+        return system_information_presenter.getPresenter()
+
+    def getObjectNavigator(self):
+        return object_navigator.getNavigator()
+
+    def getTableNavigator(self):
+        return table_navigator.getNavigator()
+
+    def getSpeechAndVerbosityManager(self):
+        return speech_and_verbosity_manager.getManager()
+
+    def getWhereAmIPresenter(self):
+        return where_am_i_presenter.getPresenter()
+
+    def getLearnModePresenter(self):
+        return learn_mode_presenter.getPresenter()
+
+    def getActionPresenter(self):
+        return action_presenter.getPresenter()
+
+    def getSleepModeManager(self):
+        return sleep_mode_manager.getManager()
+
+    def getMouseReviewer(self):
+        return mouse_review.getReviewer()
+
+    def getEventSynthesizer(self):
+        return ax_event_synthesizer.getSynthesizer()
 
     def getBookmarks(self):
         """Returns the "bookmarks" class for this script.
@@ -266,7 +292,7 @@ class Script:
         - script: the script.
         """
 
-        _eventManager.registerScriptListeners(self)
+        event_manager.getManager().registerScriptListeners(self)
 
     def deregisterEventListeners(self):
         """Tells the event manager to stop listening for all the event types
@@ -276,16 +302,7 @@ class Script:
         - script: the script.
         """
 
-        _eventManager.deregisterScriptListeners(self)
-
-    # [[[WDW - There is a circular reference going on somewhere (see
-    # bug 333168).  In the presence of this reference, the existence
-    # of a __del__ method prevents the garbage collector from
-    # collecting this object. So, we will not define a __del__ method
-    # until we understand where the circular reference is coming from.
-    #
-    #def __del__(self):
-    #    debug.println(debug.LEVEL_FINE, "DELETE SCRIPT: %s" % self.name)
+        event_manager.getManager().deregisterScriptListeners(self)
 
     def processObjectEvent(self, event):
         """Processes all AT-SPI object events of interest to this
@@ -300,25 +317,6 @@ class Script:
         - event: the Event
         """
 
-        try:
-            role = event.source.getRole()
-        except (LookupError, RuntimeError):
-            msg = 'ERROR: Exception getting role for %s' % event.source
-            debug.println(debug.LEVEL_INFO, msg, True)
-            return
-
-        # Check to see if we really want to process this event.
-        #
-        processEvent = (orca_state.activeScript == self \
-                        or self.presentIfInactive)
-        if role == pyatspi.ROLE_PROGRESS_BAR \
-           and not processEvent \
-           and settings.progressBarVerbosity == settings.PROGRESS_BAR_ALL:
-            processEvent = True
-
-        if not processEvent:
-            return
-
         if self.skipObjectEvent(event):
             return
 
@@ -340,6 +338,35 @@ class Script:
             if event.type.startswith(key):
                 self.listeners[key](event)
 
+    def _getQueuedEvent(self, eventType, detail1=None, detail2=None, any_data=None):
+        cachedEvent, eventTime = self.eventCache.get(eventType, [None, 0])
+        if not cachedEvent:
+            tokens = ["SCRIPT: No queued event of type", eventType]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            return None
+
+        if detail1 is not None and detail1 != cachedEvent.detail1:
+            tokens = ["SCRIPT: Queued event's detail1 (", cachedEvent.detail1,
+                      ") doesn't match", detail1]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            return None
+
+        if detail2 is not None and detail2 != cachedEvent.detail2:
+            tokens = ["SCRIPT: Queued event's detail2 (", cachedEvent.detail2,
+                      ") doesn't match", detail2]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            return None
+
+        if any_data is not None and any_data != cachedEvent.any_data:
+            tokens = ["SCRIPT: Queued event's any_data (",
+                      cachedEvent.any_data, ") doesn't match", any_data]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            return None
+
+        tokens = ["SCRIPT: Found matching queued event:", cachedEvent]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        return cachedEvent
+
     def skipObjectEvent(self, event):
         """Gives us, and scripts, the ability to decide an event isn't
         worth taking the time to process under the current circumstances.
@@ -380,151 +407,15 @@ class Script:
             reason = "appears to be duplicate state-changed event"
 
         if skip:
-            eventDetails = '           %s' % str(cachedEvent).replace('\t', ' ' * 11)
-            msg = 'SCRIPT: Skipping object event due to %s\n%s' % (reason, eventDetails)
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["SCRIPT: Skipping object event:", reason, cachedEvent]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         return skip
 
-    def checkKeyboardEventData(self, keyboardEvent):
-        """Checks the data on the keyboard event.
-
-        Some toolkits don't fill all the key event fields, and/or fills
-        them out with unexpected data. This method tries to fill in the
-        missing fields and validate/standardize the data we've been given.
-        While any script can override this method, it is expected that
-        this will only be done at the toolkit script level.
-
-        Arguments:
-        - keyboardEvent: an instance of input_event.KeyboardEvent
-        """
-
-        try:
-            from gi.repository import Gdk
-            keyboardEvent.keyval_name = Gdk.keyval_name(keyboardEvent.id)
-        except:
-            msg = 'ERROR: Exception getting keyval_name for id: %d' % keyboardEvent.id
-            debug.println(debug.LEVEL_INFO, msg, True)
-
-    def consumesKeyboardEvent(self, keyboardEvent):
-        """Called when a key is pressed on the keyboard.
-
-        Arguments:
-        - keyboardEvent: an instance of input_event.KeyboardEvent
-
-        Returns True if the event is of interest.
-        """
-
-        user_bindings = None
-        user_bindings_map = settings.keyBindingsMap
-        if self.__module__ in user_bindings_map:
-            user_bindings = user_bindings_map[self.__module__]
-        elif "default" in user_bindings_map:
-            user_bindings = user_bindings_map["default"]
-
-        consumes = False
-        self._lastCommandWasStructNav = False
-        if user_bindings:
-            handler = user_bindings.getInputHandler(keyboardEvent)
-            if handler \
-                 and handler.function in self.structuralNavigation.functions:
-                consumes = self.useStructuralNavigationModel()
-                if consumes:
-                    self._lastCommandWasStructNav = True
-            else:
-                consumes = handler is not None
-        if not consumes:
-            handler = self.keyBindings.getInputHandler(keyboardEvent)
-            if handler \
-                 and handler.function in self.structuralNavigation.functions:
-                consumes = self.useStructuralNavigationModel()
-                if consumes:
-                    self._lastCommandWasStructNav = True
-            else:
-                consumes = handler is not None
-        return consumes
-
-    def consumesBrailleEvent(self, brailleEvent):
-        """Called when a key is pressed on the braille display.
-
-        Arguments:
-        - brailleEvent: an instance of input_event.KeyboardEvent
-
-        Returns True if the event is of interest.
-        """
-        user_bindings = None
-        user_bindings_map = settings.brailleBindingsMap
-        if self.__module__ in user_bindings_map:
-            user_bindings = user_bindings_map[self.__module__]
-        elif "default" in user_bindings_map:
-            user_bindings = user_bindings_map["default"]
-
-        command = brailleEvent.event["command"]
-        consumes = False
-        if user_bindings:
-            consumes = command in user_bindings
-        if not consumes:
-            consumes = command in self.brailleBindings
-        return consumes
-
-    def processBrailleEvent(self, brailleEvent):
-        """Called whenever a key is pressed on the Braille display.
-
-        This method will primarily use the brailleBindings field of
-        this script instance see if this script has an interest in the
-        event.
-
-        NOTE: there is latent, but unsupported, logic for allowing
-        the user's user-settings.py file to extend and/or override
-        the brailleBindings for a script.
-
-        Arguments:
-        - brailleEvent: an instance of input_event.BrailleEvent
-        """
-
-        # We'll annotate the event with a reference to this script.
-        # This will allow external scripts to muck with the script
-        # instance if they wish.
-        #
-        brailleEvent.script = self
-
-        # We'll let the user bindings take precedence.  First, we'll
-        # check to see if they have bindings specific for the particular
-        # application, then we'll check to see if they have any default
-        # bindings to use.
-        #
-        # [[[TODO: WDW - for performance, these bindings should probably
-        # be conflated at initialization time.]]]
-        #
-        consumed = False
-        user_bindings = None
-        command = brailleEvent.event["command"]
-
-        user_bindings_map = settings.brailleBindingsMap
-        if self.name in user_bindings_map:
-            user_bindings = user_bindings_map[self.name]
-        elif "default" in user_bindings_map:
-            user_bindings = user_bindings_map["default"]
-
-        if user_bindings and command in user_bindings:
-            handler = user_bindings[command]
-            consumed = handler.processInputEvent(self, brailleEvent)
-
-        if (not consumed) and command in self.brailleBindings:
-            handler = self.brailleBindings[command]
-            consumed = handler.processInputEvent(self, brailleEvent)
-
-        return consumed
-
     def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
-        """Called when the visual object with focus changes.
-
-        The primary purpose of this method is to present locus of focus
-        information to the user.
-
-        NOTE: scripts should not call this method directly.  Instead,
-        a script should call orca.setLocusOfFocus, which will eventually
-        result in this method being called.
+        """Updates state and presents changes to the user in response to a
+        notification from the FocusManager. Do not call this method outside
+        of the FocusManager.
 
         Arguments:
         - event: if not None, the Event that caused the change
@@ -544,6 +435,11 @@ class Script:
         """
         return True
 
+    def forceScriptActivation(self, event):
+        """Allows scripts to insist that they should become active."""
+
+        return False
+
     def activate(self):
         """Called when this script is activated."""
         pass
